"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityRepositoryDataAdapter = void 0;
const lodash_1 = __importDefault(require("lodash"));
const uuid_1 = require("uuid");
const types_1 = require("./types");
const data_adapter_1 = require("./data-adapter");
class EntityRepositoryDataAdapter extends data_adapter_1.DataAdapterBase {
    constructor(socket, repository) {
        super();
        this.socket = socket;
        this.repository = repository;
        this.clientId = uuid_1.v4();
    }
    refById(id) {
        const entity = new this.repository.entityClass();
        lodash_1.default.set(entity, types_1.EntityProperty.ID, id);
        const onDataChange = (event) => {
            if (event.clientId === this.clientId) {
                console.log('We received our own feedback update, ignoring...');
                // We dont process updates made from our own client
                return;
            }
            lodash_1.default.set(entity, types_1.DataChangeKeys.LivePropertyDisabled, !event.clientId);
            lodash_1.default.assign(entity, event.data);
            lodash_1.default.unset(entity, types_1.DataChangeKeys.LivePropertyDisabled);
        };
        const eventName = this.getEventName(id);
        this.repository.findById(id)
            .then(data => {
            // Update the Entity
            lodash_1.default.assign(entity, data);
            // We dont sign up until after we received the first one
            this.socket.on(eventName, onDataChange);
            entity.change.on(types_1.EntityEvent.DataChange, (data) => {
                const message = {
                    event: types_1.EntityEvent.DataChange,
                    clientId: this.clientId,
                    model: this.repository.entityClass.modelName,
                    id,
                    data
                };
                console.log(`Emitting Event from Client: ${eventName}`);
                this.socket.emit(eventName, message);
            });
        });
        entity.destroy = () => {
            this.socket.off(eventName, onDataChange);
        };
        return entity;
    }
    getEventName(id) {
        return [
            types_1.EntityEvent.DataChange,
            this.repository.entityClass.modelName,
            id
        ].join(types_1.IdentifierSeperator);
    }
}
exports.EntityRepositoryDataAdapter = EntityRepositoryDataAdapter;
//# sourceMappingURL=entity-repository-data-adapter.js.map