"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LivePropertyDecorator = void 0;
const lodash_1 = __importDefault(require("lodash"));
const moment_1 = __importDefault(require("moment"));
const types_1 = require("../common/types");
function LivePropertyDecorator() {
    return function (target, propertyKey, descriptor) {
        const notifyFn = (target, propertyKey, value) => {
            target.onPropertyChange(propertyKey, value);
        };
        const fieldKey = `_${propertyKey}`;
        Object.defineProperty(target, propertyKey, {
            get() {
                const value = lodash_1.default.get(this, fieldKey);
                // console.log(`Live Property GET (${fieldKey}:${value})`);
                return value;
            },
            set(newValue) {
                const currentValue = lodash_1.default.get(this, fieldKey);
                if (currentValue === newValue) {
                    // We ignore updates to equivalent values
                    return;
                }
                console.log(`Live Property SET (${propertyKey}:${newValue})`);
                lodash_1.default.set(this, fieldKey, newValue);
                lodash_1.default.set(this, types_1.EntityProperty.LastUpdated, moment_1.default().toDate());
                if (!lodash_1.default.get(this, types_1.DataChangeKeys.LivePropertyDisabled)) {
                    notifyFn(this, propertyKey, newValue);
                }
            },
            enumerable: true,
            configurable: true
        });
    };
}
exports.LivePropertyDecorator = LivePropertyDecorator;
//# sourceMappingURL=live-property.js.map