/// <reference types="node" />
import { EventEmitter } from 'events';
export interface Identifiable {
    id: string;
}
export interface Destroyable {
    destroy(): void;
}
export interface LivePropertyTarget {
    onPropertyChange: (propertyKey: string, value: any | null) => void;
}
export declare abstract class Model {
    static get modelName(): string;
    static definition: ModelDefinition;
}
export interface ModelDefinition {
    readonly name: string;
}
export declare enum EntityProperty {
    ID = "id",
    LastUpdated = "lastUpdated"
}
export declare enum EntityEvent {
    DataChange = "dataChange"
}
export declare abstract class Entity extends Model implements Identifiable, Destroyable, LivePropertyTarget {
    change: EventEmitter;
    id: string;
    lastUpdated: Date;
    constructor();
    destroy(): void;
    onPropertyChange(propertyKey: string, value: any | null): void;
}
export interface EntityRepository<T extends Model, ID = {}, Relations = {}> {
    entityClass: typeof Entity & {
        prototype: T;
    };
    findById(id: string): Promise<T>;
}
export interface DataChangeMessage {
    event: string;
    model: string;
    id: string;
    data: Partial<Entity>;
    clientId?: string;
}
export declare class DataChange {
}
export declare enum DataChangeKeys {
    LivePropertyDisabled = "$$livePropertyDisabled"
}
export declare const IdentifierSeperator = ":";
