"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoopbackCrudControllerDataAdapter = void 0;
const lodash_1 = __importDefault(require("lodash"));
const uuid_1 = require("uuid");
const common_1 = require("../common");
class LoopbackCrudControllerDataAdapter extends common_1.DataAdapterBase {
    constructor(socket, entityClass, controller) {
        super();
        this.socket = socket;
        this.entityClass = entityClass;
        this.controller = controller;
        this.clientId = uuid_1.v4();
    }
    refById(id) {
        const entity = new this.entityClass();
        lodash_1.default.set(entity, common_1.EntityProperty.ID, id);
        const onDataChange = (event) => {
            console.log(`Receiving DataEvent on Client: ${event.event}`);
            if (event.clientId === this.clientId) {
                console.log('We received our own feedback update, ignoring...');
                // We dont process updates made from our own client
                return;
            }
            lodash_1.default.set(entity, common_1.DataChangeKeys.LivePropertyDisabled, !!event.clientId);
            lodash_1.default.assign(entity, event.data);
            lodash_1.default.unset(entity, common_1.DataChangeKeys.LivePropertyDisabled);
        };
        const eventName = this.getEventName(id);
        this.controller.findById({ id })
            .subscribe(data => {
            // Update the Entity
            lodash_1.default.assign(entity, data);
            // We dont sign up until after we received the first one
            console.log(`Subscribing to Events from Server: ${eventName}`);
            this.socket.on(eventName, onDataChange);
            entity.change.on(common_1.EntityEvent.DataChange, (data) => {
                const message = {
                    event: common_1.EntityEvent.DataChange,
                    clientId: this.clientId,
                    model: this.entityClass.modelName,
                    id,
                    data
                };
                console.log(`Emitting Event from Client: ${eventName}`);
                this.socket.emit(eventName, message);
            });
        });
        entity.destroy = () => {
            this.socket.off(eventName, onDataChange);
        };
        return entity;
    }
    getEventName(id) {
        return [
            common_1.EntityEvent.DataChange,
            this.entityClass.modelName,
            id
        ].join(common_1.IdentifierSeperator);
    }
}
exports.LoopbackCrudControllerDataAdapter = LoopbackCrudControllerDataAdapter;
//# sourceMappingURL=loopback-crud-controller-data-adapter.js.map